--
-- localbus_pkg.vhd - package "localbus"
--
-- SYNTHESIZABLE
--
-- Package of local bus components.
--

library ieee;
use ieee.std_logic_1164.all;

package localbus is

    -- Demand-mode DMA state machine
    -- All signals active high
    component plxddsm
        port(
            clk:           in    std_logic;  -- Local bus clock
            rst:           in    std_logic;  -- Should reduce during synthesis to global reset
            sr:            in    std_logic;  -- Synchronous reset (do not assert unless bus is idle)
            qlads:         in    std_logic;  -- LADS# signal qualified by address, FHOLDA and/or other decoding
            ldack:         in    std_logic;  -- LDACK#<n> from local bus
            lblast:        in    std_logic;  -- LBLAST# from local bus
            lwrite:        in    std_logic;  -- LWRITE from local bus
            ldreq:         out   std_logic;  -- To local bus LDREQ#<n> signal
            lready:        out   std_logic;  -- To local bus LREADY# signal
            lready_oe:     out   std_logic;  -- To local bus LREADY# signal
            lbterm:        out   std_logic;  -- To local bus LBTERM# signal
            lbterm_oe:     out   std_logic;  -- To local bus LBTERM# signal
            ld_oe:         out   std_logic;  -- Output enable for LD pins
            request:       in    std_logic;  -- Request for demand mode DMA
            burst:         in    std_logic;  -- Burst wanted/end burst
            transfer:      out   std_logic;  -- Data transfer on this clock edge
            decode:        out   std_logic;  -- Application should do further address decoding
            write:         out   std_logic); -- Cycle is a write cycle
    end component;

    -- Direct-master state machine
    -- All signals active high
    component plxdmsm
        port(
            clk:           in    std_logic;  -- Local bus clock
            rst:           in    std_logic;  -- Asynchronous reset, should reduce to global reset
            sr:            in    std_logic;  -- Synchronous reset, must be asserted when bus idle
            request:       in    std_logic;  -- Pulse to request a direct-master cycle
            stop:          in    std_logic;  -- Pulse to terminate burst
            transfer:      out   std_logic;  -- Asserted 1 clock after data transfer takes place
            done:          out   std_logic;  -- Asserted when machine is idle
            write:         in    std_logic;  -- Assert coincident with 'request' for a write cycle
            lready:        in    std_logic;  -- LREADY# signal (PCI9656) or LREADYO# (PCI9080) signal
            lbterm:        in    std_logic;  -- LREADY# signal (PCI9656) or LREADYO# (PCI9080) signal
            fholda:        in    std_logic;  -- From local bus arbiter, FPGA's bus grant signal
            fhold:         out   std_logic;  -- To local bus arbiter, FPGA's bus request signal
            la_oe:         out   std_logic;  -- When asserted, app should enable LA pins
            lads:          out   std_logic;  -- To local bus LADS# pin
            lads_oe:       out   std_logic;  -- When asserted, app should enable LADS_L pin
            lbe_oe:        out   std_logic;  -- When asserted, app should enable LBE_L pins
            lblast:        out   std_logic;  -- To local bus LBLAST# pin
            lblast_oe:     out   std_logic;  -- When asserted, app should enable LBLAST_L pin
            ld_oe:         out   std_logic;  -- When asserted, app should enable LD pins
            lwrite:        out   std_logic;  -- To local bus LWRITE pin
            lwrite_oe:     out   std_logic); -- When asserted, app should enable LWRITE pin
    end component;

    -- Direct-slave state machine
    -- All signals active high
    component plxdssm
        port(
            clk:           in    std_logic;  -- Local bus clock
            rst:           in    std_logic;  -- Should reduce during synthesis to FPGA global reset
            sr:            in    std_logic;  -- Synchronous reset (should never be asserted mid-burst)
            qlads:         in    std_logic;  -- LADS# signal qualified by address, FHOLDA and/or other decoding
            lblast:        in    std_logic;  -- Local bus LBLAST# signal
            lwrite:        in    std_logic;  -- Local bus LWRITE signal
            ld_oe:         out   std_logic;  -- Output enable for local bus data pins
            lready:        out   std_logic;  -- To local bus LREADY# signal
            lready_oe:     out   std_logic;  -- To local bus LREADY# signal
            lbterm:        out   std_logic;  -- To local bus LBTERM# signal
            lbterm_oe:     out   std_logic;  -- To local bus LBTERM# signal
            transfer:      out   std_logic;  -- Data transfer on this clock edge
            decode:        out   std_logic;  -- Application should do further address decoding
            write:         out   std_logic;  -- 'lwrite' latched on 'qlads'
            ready:         in    std_logic;  -- Apply pulse when ready
            stop:          in    std_logic); -- Assert to terminate burst
    end component;

    -- Direct-slave state machine
    component plxdssmf
        port(
            clk:           in    std_logic;  -- Local bus clock
            rst:           in    std_logic;  -- Should reduce during synthesis to FPGA global reset
            sr:            in    std_logic;  -- Synchronous reset (should never be asserted mid-burst)
            qlads:         in    std_logic;  -- LADS# signal qualified by address, FHOLDA and/or other decoding
            lblast:        in    std_logic;  -- Local bus LBLAST# signal
            lwrite:        in    std_logic;  -- Local bus LWRITE signal
            eld_oe:        out   std_logic;  -- Early output enable for local bus data pins
            lready_l:      out   std_logic;  -- To local bus LREADY# signal
            lready_oe_l:   out   std_logic;  -- To local bus LREADY# signal
            lbterm_l:      out   std_logic;  -- To local bus LBTERM# signal
            lbterm_oe_l:   out   std_logic;  -- To local bus LBTERM# signal
            transfer:      out   std_logic;  -- Data transfer on this clock edge
            decode:        out   std_logic;  -- Application should do further address decoding
            write:         out   std_logic;  -- 'lwrite' latched on 'qlads'
            ready:         in    std_logic;  -- Apply pulse when ready
            stop:          in    std_logic); -- Assert to terminate burst
    end component;

end;
