--
-- dll-xpi.vhd - Example FPGA design demonstrating clock multiplication
--
-- (C) Copyright Alpha Data 2005
--
-- Example program for:
--
--   o ADP-XPI
--
-- The local bus clock is input and multiplied by 2 using a Virtex-IIPro DCM.
--
-- The 2x clock is used to clock a free running counter, which may be read
-- via by direct slave reads on the local bus (anywhere in the FPGA space).
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library work;
use work.localbus.all;

entity dll is
    port(
        lclk:             in         std_logic;
        lreset_l:         in         std_logic;
        lwrite:           in         std_logic;
        lads_l:           in         std_logic;
        lblast_l:         in         std_logic;
        lbterm_l:         out        std_logic;
        lad:              inout      std_logic_vector(31 downto 0);
        lready_l:         out        std_logic;
        lbe_l:            in         std_logic_vector(3 downto 0);
        fholda:           in         std_logic;
        mclk:             in         std_logic);        
end dll;

architecture mixed of dll is
    
    signal rst:          std_logic;
    signal lads_i:       std_logic;
    signal lblast_i:     std_logic;
    signal lwrite_i:     std_logic;
    signal lbe_i:        std_logic_vector(3 downto 0);
    signal la_i:         std_logic_vector(24 downto 2);
    signal ld_i:         std_logic_vector(31 downto 0);
    signal ld_o:         std_logic_vector(31 downto 0);
    signal ld_oe:        std_logic;
    signal lbterm_o:     std_logic;
    signal lbterm_oe:    std_logic;
    signal lready_o:     std_logic;
    signal lready_oe:    std_logic;
    
    signal clk:          std_logic;
    signal multclk:      std_logic;
    signal locked:       std_logic;
    
    signal qlads:        std_logic;

    signal ds_xfer:      std_logic;
    signal ds_decode:    std_logic;
    signal ds_write:     std_logic;

    signal count:        std_logic_vector(31 downto 0);
    signal count_q:      std_logic_vector(31 downto 0);
    signal oe_reg0:      std_logic;
    
    signal logic0:       std_logic;
    signal logic1:       std_logic;
    
    component clocks
        port(
            rst:           in    std_logic;
            lclk:          in    std_logic;
            mclk:          in    std_logic;
            clk:           out   std_logic;
            multclk:       out   std_logic;
            locked:        out   std_logic);
    end component;

begin

    logic0 <= '0';
    logic1 <= '1';

    --
    -- Convert the inputs to active high.
    --	
    rst      <= not lreset_l;
    lblast_i <= not lblast_l;
    lads_i   <= not lads_l;
    lwrite_i <= lwrite;
    la_i     <= lad(24 downto 2);
    ld_i     <= lad;
    lbe_i    <= not lbe_l;
    
    --
    -- Generate a qualified version of 'lads_l', which is
    -- asserted when the FPGA is addressed AND the FPGA is
    -- not the local bus master.
    --
    qlads <= lads_i and not la_i(24) and not fholda;

    --
    -- 'lbterm_l' should only be driven when the FPGA is addressed; otherwise
    -- float, because the control logic on the card might also drive it.
    --
    lbterm_l <= not lbterm_o when lbterm_oe = '1' else 'Z';
    
    --
    -- 'lready_l' should only be driven when the FPGA is addressed; otherwise
    -- float because the control logic on the card might also drive it.
    --
    lready_l <= not lready_o when lready_oe = '1' else 'Z';
    
    --
    -- Drive the local data bus on a read.
    --
    lad <= ld_o when ld_oe = '1' else (others => 'Z');
    
    --
    -- Generate the 'oe_reg0' signal, for enabling the cycle count register onto
    -- the internal data bus.
    --
    -- 'oe_reg0' is set to 1 when the 'ds_decode' pulse occurs and the local bus
    -- cycle is a read.
    --
    -- 'oe_reg0' is cleared to 0 when the local bus cycle ends (signified by
    -- assertion of 'lblast_l').
    --
    generate_oe_reg : process(rst, clk)
    begin
        if rst = '1' then
            oe_reg0 <= '0';
        elsif clk'event and clk = '1' then
            if ds_xfer = '1' and (lblast_i = '1' or lbterm_o = '1') then
                oe_reg0 <= '0';
            else
                if ds_decode = '1' and ds_write = '0' then
                    oe_reg0 <= '1';
                end if;
            end if;
        end if;
    end process;

    --
    -- Generate the internal output data bus - in this case trivially.
    --
    ld_o <= count_q when oe_reg0 = '1' else (others => 'Z');

    --
    -- Cycle counter, clocked by LCLK*2
    --
    cycle_counter: process(multclk, rst)
    begin
        if rst = '1' then
            count <= (others => '0');
        elsif multclk'event and multclk = '1' then
            if ds_xfer = '1' and ds_write = '1' then
                for i in 0 to 3 loop
                    if lbe_i(i) = '1' then
                        count(8 * (i + 1) - 1 downto 8 * i) <=
                            ld_i(8 * (i + 1) - 1 downto 8 * i);
                    end if;
                end loop;
            else
                count <= count + '1';
            end if;
        end if;
    end process;

    --
    -- Resample the LCLK*2 cycle counter to LCLK. Should be safe with
    -- regard to metastability because the two clock domains are
    -- phase-locked.
    --
    resample: process(clk, rst)
    begin
        if rst = '1' then
            count_q <= (others => '0');
        elsif clk'event and clk = '1' then
            count_q <= count;
        end if;
    end process;      

    --
    -- Instantiate the clocks module, which generates LCLK*2.
    --
    clocks0: clocks
        port map(
            rst     => rst,
            lclk    => lclk,
            mclk    => mclk,
            clk     => clk,
            multclk => multclk,
            locked  => locked);
    
    --
    -- Instantiate the direct slave state machine; monitors the local bus for
    -- direct slave cycles and responds appropriately.
    --
    plxdssm0: plxdssm
        port map(
            clk        => clk,
            rst        => rst,
            sr         => logic0,
            qlads      => qlads,
            lblast     => lblast_i,
            lwrite     => lwrite_i,
            ld_oe      => ld_oe,
            lready     => lready_o,
            lready_oe  => lready_oe,
            lbterm     => lbterm_o,
            lbterm_oe  => lbterm_oe,
            transfer   => ds_xfer,
            decode     => ds_decode,
            write      => ds_write,
            ready      => logic1,
            stop       => logic1);
    
end mixed;
