--
-- simple-wrc2.vhd - Trivial design demonstrating how to implement
--                   registers in the FPGA.
--
-- (C) Copyright Alpha Data Parallel Systems Ltd. 1999-2005
--
-- Example program for:
--
--   o ADP-WRCII
--   o ADP-DRCII
--

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.localbus.all;

entity simple is
   port(
      lclk:		in	std_logic;
      lreset_l:         in	std_logic; 
      lwrite:		in	std_logic;
      lads_l:		in	std_logic;
      lblast_l:         in	std_logic;
      lbterm_l:         out	std_logic;
      la:		in      std_logic_vector(24 downto 2);
      ld:		inout	std_logic_vector(31 downto 0);
      lready_l:         out	std_logic; 
      lbe_l:		in	std_logic_vector(3 downto 0);
      fholda:		in	std_logic);
end simple;

architecture mixed of simple is
    
    signal	rst:		std_logic;
    signal	lblast_i:	std_logic;
    signal	lads_i:		std_logic;
    signal	lwrite_i:	std_logic;
    signal	lready_o:	std_logic;
    signal	lready_oe:	std_logic;
    signal	lbterm_o:	std_logic;
    signal	lbterm_oe:	std_logic;
    signal	la_i:		std_logic_vector(24 downto 2);
    signal	ld_o:		std_logic_vector(31 downto 0);
    signal	ld_i:		std_logic_vector(31 downto 0);
    signal	ld_oe:		std_logic;
    signal	lbe_i:		std_logic_vector(3 downto 0);

    signal	qlads:		std_logic;
    signal	ds_xfer:	std_logic;
    signal      ds_decode:      std_logic;    
    signal      ds_write:       std_logic;    

    signal      la_q:           std_logic_vector(24 downto 2);
    
    signal	reg0:           std_logic_vector(31 downto 0);
    signal	reg0_rev:       std_logic_vector(31 downto 0);
    signal      oe_reg0:        std_logic;
    signal	reg1:           std_logic_vector(31 downto 0);
    signal      oe_reg1:        std_logic;
    
    signal	logic0:		std_logic;
    signal	logic1:		std_logic;
    
begin

    logic0 <= '0';
    logic1 <= '1';

    --
    -- Convert the inputs to active high.
    --	
    rst      <= not lreset_l;
    lblast_i <= not lblast_l;
    lads_i   <= not lads_l;
    lwrite_i <= lwrite;
    la_i     <= la;
    ld_i     <= ld;
    lbe_i    <= not lbe_l;
    
    --
    -- Generate a qualified version of 'lads_l', which is
    -- asserted when the FPGA is addressed AND the FPGA is
    -- not the local bus master.
    --
    qlads <= lads_i and not la_i(24) and not fholda;

    --
    -- Latch the local bus address on the 'lads_l' pulse.
    --
    latch_addr : process(rst, lclk)
    begin
        if rst = '1' then
            la_q <= (others => '0');
        elsif lclk'event and lclk = '1' then
            if lads_i = '1' then
                la_q <= la_i;
            end if;
        end if;
    end process;
    
    --
    -- 'lbterm_l' should only be driven when the FPGA is addressed; otherwise
    -- float, because the control logic on the card might also drive it.
    --
    lbterm_l <= not lbterm_o when lbterm_oe = '1' else 'Z';
    
    --
    -- 'lready_l' should only be driven when the FPGA is addressed; otherwise
    -- float because the control logic on the card might also drive it.
    --
    lready_l <= not lready_o when lready_oe = '1' else 'Z';
    
    --
    -- Drive the local data bus on a read.
    --
    ld <= ld_o when ld_oe = '1' else (others => 'Z');
    
    --
    -- If the current cycle is a write, update the registers
    --		
    update_regs : process(lclk, rst)
    begin
        if rst = '1' then
            reg0 <= (others => '0');
            reg1 <= (others => '0');
        elsif lclk'event and lclk = '1' then
            if ds_xfer = '1' and ds_write = '1' then
                if la_q(2) = '0' then
                    if lbe_i(0) = '1' then
                        reg0(7 downto 0) <= ld_i(7 downto 0);
                    end if;
                    if lbe_i(1) = '1' then
                        reg0(15 downto 8) <= ld_i(15 downto 8);
                    end if;
                    if lbe_i(2) = '1' then
                        reg0(23 downto 16) <= ld_i(23 downto 16);
                    end if;
                    if lbe_i(3) = '1' then
                        reg0(31 downto 24) <= ld_i(31 downto 24);
                    end if;
                else
                    if lbe_i(0) = '1' then
                        reg1(7 downto 0) <= ld_i(7 downto 0);
                    end if;
                    if lbe_i(1) = '1' then
                        reg1(15 downto 8) <= ld_i(15 downto 8);
                    end if;
                    if lbe_i(2) = '1' then
                        reg1(23 downto 16) <= ld_i(23 downto 16);
                    end if;
                    if lbe_i(3) = '1' then
                        reg1(31 downto 24) <= ld_i(31 downto 24);
                    end if;
                end if;
            end if;
        end if;
    end process;

    --
    -- Generate the 'oe_reg*' signals, for enabling registers onto
    -- the internal data bus.
    --
    generate_oe_reg : process(rst, lclk)
    begin
        if rst = '1' then
            oe_reg0 <= '0';
            oe_reg1 <= '0';
        elsif lclk'event and lclk = '1' then
            if ds_xfer = '1' and (lblast_i = '1' or lbterm_o = '1') then
                oe_reg0 <= '0';
                oe_reg1 <= '0';
            else
                if ds_decode = '1' and ds_write = '0' then
                    oe_reg0 <= not la_q(2);
                    oe_reg1 <= la_q(2);
                end if;
            end if;
        end if;
    end process;
        
    --
    -- Drive 'ld_o' with nibble reversed 'reg0' when register 0 is read,
    -- and with 'reg1' when register 1 is read.
    --
    -- This synthesises to an internal tristate bus, unless the
    -- synthesis tool optimises it into a multiplexor.
    --
    reg0_rev <= reg0(3 downto 0) &
                reg0(7 downto 4) &
                reg0(11 downto 8) &
                reg0(15 downto 12) &
                reg0(19 downto 16) &
                reg0(23 downto 20) &
                reg0(27 downto 24) &
                reg0(31 downto 28);
    ld_o <= reg0_rev when oe_reg0 = '1' else (others => 'Z');
    ld_o <= reg1     when oe_reg1 = '1' else (others => 'Z');
    
    --
    -- Instantiate the direct slave state machine; monitors the local bus for
    -- direct slave cycles and responds appropriately.
    --
    dssm : plxdssm
        port map(
            clk        => lclk,
            rst        => rst,
            sr         => logic0,
            qlads      => qlads,
            lblast     => lblast_i,
            lwrite     => lwrite_i,
            ld_oe      => ld_oe,
            lready     => lready_o,
            lready_oe  => lready_oe,
            lbterm     => lbterm_o,
            lbterm_oe  => lbterm_oe,
            transfer   => ds_xfer,
            decode     => ds_decode,
            write      => ds_write,
            ready      => logic1,
            stop       => logic1);

end mixed;
